import socket
import logging
import threading
import time
import struct
import json
from http.server import BaseHTTPRequestHandler, HTTPServer
from socketserver import ThreadingMixIn

from .utils import ProxyUtils
from .mqtt_client import MQTTClient
from commands.command_heartbeat import CommandHeartbeat
from commands.command_01 import Command01
from commands.command_02 import Command02
from commands.command_03 import Command03
from commands.command_04 import Command04
from commands.command_05 import Command05
from commands.command_06 import Command06
from commands.command_07 import Command07
from commands.command_08 import Command08
from commands.command_09 import Command09
from commands.command_0A import Command0A
from commands.command_19_1A import Command191A
from commands.command_1F_20 import Command1F20
from commands.command_21_22 import Command2122
from commands.command_23_24 import Command2324
from commands.command_25 import Command25
from commands.command_26_27 import Command2627
from commands.command_30 import Command30
from commands.command_37_38 import Command3738

# 自定义 HTTP 服务器，支持多线程
class ThreadingHTTPServer(ThreadingMixIn, HTTPServer):
    pass

class ChargingPileDirectConnection:
    def __init__(self, host="0.0.0.0", port=52461, api_host="0.0.0.0", api_port=8083):
        logging.info(f"初始化 ChargingPileDirectConnection，host={host}, port={port}, API host={api_host}, API port={api_port}")
        self.host = host
        self.port = port
        self.api_host = api_host
        self.api_port = api_port
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        try:
            self.sock.bind((self.host, self.port))
            logging.info(f"绑定端口 {self.port} 成功")
        except Exception as e:
            logging.error(f"绑定端口失败: {str(e)}")
            raise
        self.sock.listen(1)
        self.sock.settimeout(10)
        self.running = False
        self.conn = None
        self.addr = None
        self.mqtt_client = MQTTClient()
        self.utils = ProxyUtils()
        self.command_handlers = {
            0x01: Command01(),
            0x02: Command02(),
            0x03: Command03(),
            0x04: Command04(),
            0x05: Command05(),
            0x06: Command06(),
            0x07: Command07(),
            0x08: Command08(),
            0x09: Command09(),
            0x0A: Command0A(),
            0x19: Command191A(),
            0x1A: Command191A(),
            0x1F: Command1F20(),
            0x20: Command1F20(),
            0x21: Command2122(),
            0x22: Command2122(),
            0x23: Command2324(),
            0x24: Command2324(),
            0x25: Command25(),
            0x26: Command2627(),
            0x27: Command2627(),
            0x30: Command30(),
            0x0B: CommandHeartbeat(),
            0x0C: CommandHeartbeat(),
            0x37: Command3738(),
            0x38: Command3738(),
        }
        self.heartbeat_info = {}
        self.heartbeat_thread = threading.Thread(target=self.check_heartbeat_timeout)
        self.heartbeat_thread.daemon = True
        self.send_heartbeat_thread = threading.Thread(target=self.send_heartbeat_periodically)
        self.send_heartbeat_thread.daemon = True
        self.current_pile_id = None
        self.is_logged_in = False
        self.api_server = None
        self.api_thread = None

        # 定义 HTTP 请求处理器
        class APIHandler(BaseHTTPRequestHandler):
            def do_POST(self):
                logging.info(f"收到 POST 请求: {self.path} from {self.client_address}")
                if self.path == "/send_billing_template":
                    try:
                        content_length = int(self.headers['Content-Length'])
                        post_data = self.rfile.read(content_length)
                        logging.debug(f"请求数据: {post_data.decode('utf-8')}")
                        data = json.loads(post_data.decode('utf-8'))

                        pile_id_str = data.get("pile_id")
                        billing_template_data = data.get("billing_template")

                        if not pile_id_str or not billing_template_data:
                            logging.warning("缺少 pile_id 或 billing_template")
                            self.send_response(400)
                            self.send_header('Content-Type', 'application/json')
                            self.end_headers()
                            self.wfile.write(json.dumps({"status": "error", "message": "Missing pile_id or billing_template"}).encode())
                            return

                        # 转换 pile_id 为字节
                        pile_id = bytes.fromhex(pile_id_str.replace(" ", ""))
                        if len(pile_id) != 8:
                            logging.warning(f"无效的 pile_id 长度: {pile_id_str}")
                            self.send_response(400)
                            self.send_header('Content-Type', 'application/json')
                            self.end_headers()
                            self.wfile.write(json.dumps({"status": "error", "message": "Invalid pile_id length"}).encode())
                            return

                        # 检查桩号是否匹配
                        if pile_id != self.server.direct_connection.current_pile_id:
                            logging.warning(f"桩号不匹配，请求: {pile_id_str}, 当前: {self.server.direct_connection.current_pile_id.hex().upper() if self.server.direct_connection.current_pile_id else 'None'}")
                            self.send_response(400)
                            self.send_header('Content-Type', 'application/json')
                            self.end_headers()
                            self.wfile.write(json.dumps({"status": "error", "message": f"Pile ID {pile_id_str} does not match current pile {self.server.direct_connection.current_pile_id.hex().upper() if self.server.direct_connection.current_pile_id else 'None'}"}).encode())
                            return

                        # 解析计费模版
                        billing_template = []
                        for entry in billing_template_data:
                            time_minutes = entry.get("time_minutes")
                            price_per_minute = entry.get("price_per_minute")
                            total_seconds = entry.get("total_seconds")
                            total_cost = entry.get("total_cost")
                            if not all(isinstance(x, int) for x in [time_minutes, price_per_minute, total_seconds, total_cost]):
                                logging.warning(f"无效的计费模版条目: {entry}")
                                self.send_response(400)
                                self.send_header('Content-Type', 'application/json')
                                self.end_headers()
                                self.wfile.write(json.dumps({"status": "error", "message": "Invalid billing template entry"}).encode())
                                return
                            billing_template.append((time_minutes, price_per_minute, total_seconds, total_cost))

                        # 调用下发方法
                        result = self.server.direct_connection.send_billing_template(pile_id, billing_template)
                        if result:
                            logging.info("计费模版下发成功")
                            self.send_response(200)
                            self.send_header('Content-Type', 'application/json')
                            self.end_headers()
                            self.wfile.write(json.dumps({"status": "success"}).encode())
                        else:
                            logging.error("计费模版下发失败")
                            self.send_response(500)
                            self.send_header('Content-Type', 'application/json')
                            self.end_headers()
                            self.wfile.write(json.dumps({"status": "error", "message": "Failed to send billing template"}).encode())
                    except json.JSONDecodeError as e:
                        logging.error(f"JSON 解析错误: {str(e)}")
                        self.send_response(400)
                        self.send_header('Content-Type', 'application/json')
                        self.end_headers()
                        self.wfile.write(json.dumps({"status": "error", "message": "Invalid JSON format"}).encode())
                    except Exception as e:
                        logging.error(f"处理请求时出错: {str(e)}")
                        self.send_response(500)
                        self.send_header('Content-Type', 'application/json')
                        self.end_headers()
                        self.wfile.write(json.dumps({"status": "error", "message": str(e)}).encode())
                else:
                    logging.warning(f"未知端点: {self.path}")
                    self.send_response(404)
                    self.send_header('Content-Type', 'application/json')
                    self.end_headers()
                    self.wfile.write(json.dumps({"status": "error", "message": "Endpoint not found"}).encode())

        # 启动 HTTP 服务器
        self.api_server = ThreadingHTTPServer((self.api_host, self.api_port), APIHandler)
        self.api_server.direct_connection = self  # 将实例绑定到服务器
        self.api_thread = threading.Thread(target=self.api_server.serve_forever)
        self.api_thread.daemon = True
        self.api_thread.start()
        logging.info(f"HTTP API 服务器启动，监听 {self.api_host}:{self.api_port}")

    def connect(self):
        try:
            logging.info(f"正在监听 {self.host}:{self.port} 等待充电桩连接...")
            self.conn, self.addr = self.sock.accept()
            self.conn.settimeout(10)
            self.running = True
            msg = f"成功接受充电桩连接 {self.addr}"
            logging.info(msg)
            self.heartbeat_thread.start()
            self.send_heartbeat_thread.start()
            return True
        except socket.timeout:
            logging.debug("未收到连接请求，继续监听...")
            return False
        except Exception as e:
            error_msg = f"接受连接失败: {str(e)}"
            logging.error(error_msg)
            return False

    def check_heartbeat_timeout(self):
        while self.running:
            try:
                current_time = time.time()
                disconnected_piles = []
                for pile_id, last_heartbeat in self.heartbeat_info.items():
                    if current_time - last_heartbeat > 60:
                        msg = f"充电桩 {pile_id.hex().upper()} 心跳超时 {current_time - last_heartbeat:.1f}秒"
                        logging.warning(msg)
                        disconnected_piles.append(pile_id)
                for pile_id in disconnected_piles:
                    del self.heartbeat_info[pile_id]
                time.sleep(5)
            except Exception as e:
                logging.error(f"心跳检测错误: {str(e)}")
                time.sleep(5)

    def update_heartbeat(self, pile_id):
        self.heartbeat_info[pile_id] = time.time()
        logging.debug(f"更新充电桩 {pile_id.hex().upper()} 心跳时间")

    def send_heartbeat_periodically(self):
        while self.running:
            try:
                if self.current_pile_id and self.conn:
                    handler = self.command_handlers[0x0B]
                    frame = handler.build_0b_heartbeat(self.current_pile_id)
                    self.send_frame(frame)
                    logging.debug(f"发送0BH平台心跳: {frame.hex().upper()}")
                time.sleep(5)
            except Exception as e:
                logging.error(f"发送0BH心跳失败: {str(e)}")
                time.sleep(5)

    def send_frame(self, data):
        try:
            if self.conn:
                self.conn.send(data)
                logging.info(f"发送数据: {data.hex().upper()}")
                return True
            return False
        except Exception as e:
            logging.error(f"发送数据失败: {str(e)}")
            return False

    def calculate_checksum(self, data):
        """计算校验码，从命令字节到倒数第二字节异或"""
        checksum = 0
        for b in data[2:-1]:
            checksum ^= b
        return checksum

    def receive_frame(self):
        try:
            header = self.conn.recv(14)
            logging.debug(f"收到帧头: {header.hex().upper()}")
            if len(header) < 14 or header[:2] != b'JX':
                logging.warning("无效帧头")
                return None
            data_len = struct.unpack('<H', header[12:14])[0]
            data = header + self.conn.recv(data_len + 1)
            logging.debug(f"收到完整报文: {data.hex().upper()}")
            calculated_checksum = self.calculate_checksum(data)
            if calculated_checksum != data[-1]:
                logging.warning(f"校验码不匹配，计算值: {calculated_checksum:02X}, 实际值: {data[-1]:02X}")
                return None
            return data
        except socket.timeout:
            logging.warning("接收数据超时，可能是充电桩未响应或网络问题")
            return self.receive_frame()
        except Exception as e:
            logging.error(f"接收数据失败: {str(e)}, 报文: {data.hex() if data else 'None'}")
            return None

    def handle_command(self, data):
        if not data or len(data) < 14:
            logging.debug("收到空或无效数据，跳过处理")
            return

        command = data[2]
        pile_id = data[3:11]
        self.current_pile_id = pile_id
        logging.info(f"处理命令: {command:02X}, 桩号: {pile_id.hex().upper()}")

        handler = self.command_handlers.get(command)
        if handler:
            if command == 0x01:
                logging.info("收到01H请求连接")
                response = self.command_handlers[0x02].build_02h_response(pile_id, allow=True)
                if response:
                    self.send_frame(response)
                    logging.info(f"发送02H允许连接响应: {response.hex().upper()}")
                elif command == 0x03:
                    logging.info(f"收到03H登录信息，原始报文: {data.hex().upper()}")
                    parsed_data = self.command_handlers[0x03].process_03h(data)
                    if parsed_data:
                        gun_count = parsed_data["login_data"]["gun_count"]
                        response = self.command_handlers[0x02].build_02h_response(pile_id, allow=True,
                                                                                  gun_count=gun_count)
                        if response:
                            self.send_frame(response)
                            logging.info(f"发送02H确认登录响应: {response.hex().upper()}")
                            time.sleep(5)  # 等待5秒确认是否重复03H
                            if not self.is_logged_in and self.conn:
                                logging.warning("充电桩未进入正式状态，尝试重发02H")
                                self.send_frame(response)
                        if not self.is_logged_in:
                            self.is_logged_in = True
                            logging.info(f"充电桩 {pile_id.hex().upper()} 登录成功")
                            request_08h = self.command_handlers[0x08].build_08h_request(pile_id)
                            if request_08h:
                                self.send_frame(request_08h)
                                logging.info(f"发送08H查询状态请求: {request_08h.hex().upper()}")
                            request_0a = self.command_handlers[0x0A].build_0a_request(pile_id)
                            if request_0a:
                                self.send_frame(request_0a)
                                logging.info(f"发送0A查询记录请求: {request_0a.hex().upper()}")
                    else:
                        logging.warning(f"充电桩 {pile_id.hex().upper()} 03H解析失败")
            elif command == 0x08:
                logging.info(f"收到08H响应，原始报文: {data.hex().upper()}")
                self.command_handlers[0x08].process_08h_response(data)
            elif command == 0x09:
                logging.info(f"收到09H状态信息，原始报文: {data.hex().upper()}")
                self.command_handlers[0x09].process_09h(data)
            elif command == 0x0A:
                logging.info(f"收到0A响应，原始报文: {data.hex().upper()}")
                self.command_handlers[0x0A].process_0a_response(data)
            elif command == 0x19:
                logging.info(f"收到19H卡鉴权请求，原始报文: {data.hex().upper()}")
                self.command_handlers[0x19].process_and_respond(data, self.conn)
                if self.is_logged_in:
                    self.start_charging(pile_id)
            elif command == 0x1F:
                logging.info(f"收到1FH启动充电命令，原始报文: {data.hex().upper()}")
                self.command_handlers[0x1F].process_1f(pile_id, self.conn)
            elif command == 0x20:
                logging.info(f"收到20H启动充电结果，原始报文: {data.hex().upper()}")
                self.command_handlers[0x20].parse_20h(data)
            elif command == 0x21:
                logging.info(f"收到21H充电状态，原始报文: {data.hex().upper()}")
                self.command_handlers[0x21].process_and_respond(data, self.conn)
            elif command == 0x23:
                logging.info(f"收到23H停止充电命令，原始报文: {data.hex().upper()}")
                self.command_handlers[0x23].process_and_respond(data, self.conn)
            elif command == 0x25:
                logging.info(f"收到25H充电记录，原始报文: {data.hex().upper()}")
                self.command_handlers[0x25].process_25h(data)
            elif command == 0x37:
                logging.info(f"发送37H下发计费模版请求，原始报文: {data.hex().upper()}")
            elif command == 0x38:
                logging.info(f"收到38H计费模版下发结果，原始报文: {data.hex().upper()}")
                self.command_handlers[0x38].parse_38h_response(data)
            elif command == 0x0B:
                self.update_heartbeat(pile_id)
                logging.info(f"发送0BH平台心跳响应: {data.hex().upper()}")
                response = self.command_handlers[0x0C].build_0c_heartbeat(pile_id)
                if response:
                    self.send_frame(response)
            elif command == 0x0C:
                self.update_heartbeat(pile_id)
                logging.info(f"收到0CH充电桩心跳: {data.hex().upper()}")
                self.command_handlers[0x0C].process_0c_heartbeat(data, self.conn)
            elif command == 0x04:
                self.command_handlers[0x04].process_04h(data)
                self.stop()
            elif command == 0x05:
                self.command_handlers[0x05].process_05h_response(data)
                response = self.command_handlers[0x06].build_06h_response(pile_id)
                if response:
                    self.send_frame(response)
            elif command == 0x06:
                self.command_handlers[0x06].process_06h(data)
            elif command == 0x07:
                self.command_handlers[0x07].process_07h(data, self.conn)
            elif command == 0x26:
                self.command_handlers[0x26].build_26h_command(data, self.conn)
            elif command == 0x27:
                self.command_handlers[0x27].parse_27h(data)
            elif command == 0x30:
                self.command_handlers[0x30].parse_30h_bms_status(data)
            else:
                logging.warning(f"未知命令: {command:02X}, 数据: {data.hex().upper()}")
        else:
            logging.warning(f"未找到命令 {command:02X} 的处理程序")

    def start_charging(self, pile_id):
        if not self.is_logged_in:
            logging.error("充电桩未登录，无法启动充电")
            return False
        if not self.conn:
            logging.error("未建立连接，无法启动充电")
            return False
        handler = self.command_handlers[0x1F]
        result = handler.process_1f(pile_id, self.conn)
        if result:
            logging.info(f"成功发送1FH启动充电命令给桩号 {pile_id.hex().upper()}")
        return result

    def send_billing_template(self, pile_id, billing_template):
        if not self.is_logged_in:
            logging.error("充电桩未登录，无法下发计费模版")
            return False
        if not self.conn:
            logging.error("未建立连接，无法下发计费模版")
            return False
        handler = self.command_handlers[0x37]
        frame = handler.build_37h_request(pile_id, billing_template)
        if frame and self.send_frame(frame):
            logging.info(f"成功发送37H下发计费模版请求给桩号 {pile_id.hex().upper()}")
            return True
        return False

    def run(self):
        self.running = True
        while self.running:
            if not self.connect():
                continue
            try:
                while self.running and self.conn:
                    data = self.receive_frame()
                    if data:
                        self.handle_command(data)
                    time.sleep(0.1)
            except Exception as e:
                logging.error(f"连接处理出错: {str(e)}")
            finally:
                self.is_logged_in = False
                if self.conn:
                    self.conn.close()
                    self.conn = None
                self.current_pile_id = None
                logging.info("连接断开，等待新的连接...")

    def stop(self):
        self.running = False
        if self.conn:
            self.conn.close()
        if self.sock:
            self.sock.close()
        if self.api_server:
            self.api_server.shutdown()
            self.api_server.server_close()
        logging.info("直连程序停止")

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG, format='%(asctime)s - %(levelname)s - %(message)s')
    connection = ChargingPileDirectConnection(host="0.0.0.0", port=52461, api_host="0.0.0.0", api_port=8083)
    connection.run()