import paho.mqtt.client as mqtt
import json
import logging
import taosrest
from datetime import datetime

# 配置日志
logging.basicConfig(
    filename='charge_order_jiuxing.log',
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    encoding='utf-8'
)

class ChargeOrderJiuxingParser:
    def __init__(self, mqtt_host="123.6.102.119", mqtt_port=1883, mqtt_username="emqx_test", mqtt_password="emqx_test",
                 tdengine_url="http://123.6.102.119:6041", tdengine_user="readonly_user", tdengine_password="Aassword123"):
        # MQTT 配置
        self.mqtt_client = mqtt.Client(client_id="ChargeOrderJiuxingParser", protocol=mqtt.MQTTv311,
                                       callback_api_version=mqtt.CallbackAPIVersion.VERSION2)
        self.mqtt_client.username_pw_set(mqtt_username, mqtt_password)
        self.mqtt_client.on_connect = self.on_connect
        self.mqtt_client.on_message = self.on_message
        self.mqtt_host = mqtt_host
        self.mqtt_port = mqtt_port
        self.connected = False

        # TDengine 配置
        self.tdengine_url = tdengine_url
        self.tdengine_user = tdengine_user
        self.tdengine_password = tdengine_password
        self.tdengine_conn = None

    def connect(self):
        """连接 MQTT 和 TDengine"""
        # 连接 MQTT
        try:
            self.mqtt_client.connect(self.mqtt_host, self.mqtt_port, 60)
            self.mqtt_client.loop_start()
            logging.info("Connected to MQTT broker")
        except Exception as e:
            logging.error(f"MQTT connection error: {str(e)}")
            raise

        # 连接 TDengine
        try:
            self.tdengine_conn = taosrest.connect(
                url=self.tdengine_url,
                user=self.tdengine_user,
                password=self.tdengine_password,
                database="tms_design"
            )
            logging.info("Connected to TDengine via REST")
        except Exception as e:
            logging.error(f"TDengine connection error: {str(e)}")
            raise

    def on_connect(self, client, userdata, flags, rc, properties=None):
        if rc == 0:
            self.connected = True
            self.mqtt_client.subscribe("hejin/charging/log", qos=1)
            logging.info("Subscribed to hejin/charging/log")
        else:
            logging.error(f"Failed to connect to MQTT broker with code: {rc}")

    def on_message(self, client, userdata, msg, properties=None):
        """处理接收到的 MQTT 消息"""
        try:
            payload = msg.payload.decode('utf-8')
            data = json.loads(payload)
            logging.info(f"Received message: {data}")

            # 解析报文并存储
            sql = self.parse_and_generate_sql(data)
            if sql:
                self.tdengine_conn.execute(sql)
                logging.info(f"Inserted into TDengine: {sql}")
        except Exception as e:
            logging.error(f"Error processing message: {str(e)}")

    def hex_to_ascii(self, hex_str):
        """将 HEX 字符串转为 ASCII"""
        try:
            return bytes.fromhex(hex_str).decode('ascii').strip('\x00')
        except:
            return ""

    def parse_bcd_time(self, bcd):
        """解析 BCD 码时间（格式：YYMMDDHHMMSS）"""
        try:
            year = f"20{bcd[0:2]}"  # 假设 19 表示 2019
            month = bcd[2:4]
            day = bcd[4:6]
            hour = bcd[6:8]
            minute = bcd[8:10]
            second = bcd[10:12]
            return f"{year}-{month}-{day}T{hour}:{minute}:{second}+08:00"
        except:
            return None

    def parse_and_generate_sql(self, data):
        """解析报文并生成插入 SQL"""
        hex_data = data[3].replace(" ", "")  # 移除空格
        pile_id = data[4]  # 桩号
        time = data[2].replace(" ", "T") + "+08:00"  # 时间

        # 提取基本字段
        if len(hex_data) < 30:  # 最小长度：14字节（HEX表示为28字符）+校验码(2)
            logging.warning(f"数据包长度不足: {hex_data}")
            return None

        company = hex_data[0:4]  # 4A58
        if company != "4A58":
            logging.warning(f"无效帧起始: {company}")
            return None

        cmd = hex_data[4:6]  # 命令码
        length_str = hex_data[24:28]  # 数据域长度
        length = int(length_str, 16) * 2  # HEX字符数
        data_domain = hex_data[28:28 + length] if length > 0 else ""

        # 从桩号提取信息
        operator_id = pile_id[0:4]  # 运营商编号
        station_no = pile_id[6:12]  # 站点编号
        equip_no = pile_id[12:16]  # 站内桩地址（作为充电机编号）

        # 初始化默认值
        order_sn = "unknown"
        start_time = time
        stop_time = None  # 充电结束时间未知
        start_soc = 0.0
        stop_soc = 0.0
        elect_amount = 0.0
        chg_time = 0
        battery_id = "unknown"
        plate_no = "unknown"
        vin = "unknown"
        stop_reason = 0
        sharp_elect = 0.0
        peak_elect = 0.0
        valley_elect = 0.0
        total_money = 0.0
        elect_money = 0.0
        service_money = 0.0
        battery_money = 0.0
        rate_model_id = "unknown"
        exchange_order_sn = "unknown"
        chg_switch = 0
        start_type = 0
        created_at = time
        flat_elect = 0.0

        # 根据命令码解析数据域
        if cmd == "0B":  # 平台心跳
            if len(data_domain) >= 12:  # 6字节时间 + 1字节超时次数
                time_str = data_domain[0:12]  # BCD码时间
                parsed_time = self.parse_bcd_time(time_str)
                if parsed_time:
                    start_time = parsed_time
                    created_at = parsed_time

        elif cmd == "25":  # 充电信息（表 3.9.9）
            if len(data_domain) >= 92:  # 确保数据域足够长
                # 充电用户编号（字节 0-15，ASCII）
                battery_id = self.hex_to_ascii(data_domain[0:32])
                # 充电电压（字节 16-17，HEX，单位 0.1V）
                voltage = int(data_domain[32:36], 16) / 10.0
                # 充电电流（字节 18-19，HEX，单位 0.1A）
                current = int(data_domain[36:40], 16) / 10.0
                # 充电电量（字节 20-21，HEX，单位 0.01kWh）
                elect_amount = round(int(data_domain[40:44], 16) / 100.0, 2)
                # SOC（字节 22，HEX，单位 %）
                start_soc = int(data_domain[44:46], 16)
                # 故障状态（字节 23-26，HEX）
                fault_status = data_domain[46:54]
                # 充电状态（字节 27，HEX）
                charge_status = int(data_domain[54:56], 16)
                # 充电开始时间（字节 28-33，BCD码）
                start_time_str = data_domain[56:68]
                start_time = self.parse_bcd_time(start_time_str) or start_time
                # 充电结束时间（字节 34-39，BCD码）
                stop_time_str = data_domain[68:80]
                stop_time = self.parse_bcd_time(stop_time_str)
                # 充电时长（字节 40-41，HEX，单位 秒）
                chg_time = int(data_domain[80:84], 16)
                # 停止原因（字节 42，HEX）
                stop_reason = int(data_domain[84:86], 16)
                # 尖时段电量（字节 43-44，HEX，单位 0.01kWh）
                sharp_elect = round(int(data_domain[86:90], 16) / 100.0, 2)
                # 峰时段电量（字节 45-46，HEX，单位 0.01kWh）
                peak_elect = round(int(data_domain[90:94], 16) / 100.0, 2)
                # 谷时段电量（字节 47-48，HEX，单位 0.01kWh）
                valley_elect = round(int(data_domain[94:98], 16) / 100.0, 2)
                # 总费用（字节 49-50，HEX，单位 0.01元）
                total_money = round(int(data_domain[98:102], 16) / 100.0, 2)
                # 电费（字节 51-52，HEX，单位 0.01元）
                elect_money = round(int(data_domain[102:106], 16) / 100.0, 2)
                # 服务费（字节 53-54，HEX，单位 0.01元）
                service_money = round(int(data_domain[106:110], 16) / 100.0, 2)
                # 平时段电量（字节 55-56，HEX，单位 0.01kWh）
                flat_elect = round(int(data_domain[110:114], 16) / 100.0, 2)

        else:
            logging.info(f"未处理的命令码: {cmd}")
            return None  # 跳过未处理的命令

        # 构建插入 SQL
        stop_time_sql = f"'{stop_time}'" if stop_time else "NULL"
        sql = (
            "INSERT INTO charge_order_jiuxing (order_sn, operator_id, station_no, equip_no, start_time, stop_time, "
            "start_soc, stop_soc, elect_amount, chg_time, battery_id, plate_no, vin, stop_reason, sharp_elect, peak_elect, "
            "valley_elect, total_money, elect_money, service_money, battery_money, rate_model_id, exchange_order_sn, "
            "chg_switch, start_type, created_at, flat_elect) VALUES "
            f"('{order_sn}', '{operator_id}', '{station_no}', '{equip_no}', '{start_time}', {stop_time_sql}, "
            f"{start_soc}, {stop_soc}, {elect_amount}, {chg_time}, '{battery_id}', '{plate_no}', '{vin}', {stop_reason}, "
            f"{sharp_elect}, {peak_elect}, {valley_elect}, {total_money}, {elect_money}, {service_money}, {battery_money}, "
            f"'{rate_model_id}', '{exchange_order_sn}', {chg_switch}, {start_type}, '{created_at}', {flat_elect})"
        )
        return sql

    def run(self):
        """启动程序"""
        self.connect()
        try:
            while True:
                pass  # 保持程序运行
        except KeyboardInterrupt:
            self.mqtt_client.loop_stop()
            self.mqtt_client.disconnect()
            if self.tdengine_conn:
                self.tdengine_conn.close()
            logging.info("Program stopped")

if __name__ == "__main__":
    parser = ChargeOrderJiuxingParser(
        mqtt_host="123.6.102.119",
        mqtt_port=1883,
        mqtt_username="emqx_test",
        mqtt_password="emqx_test",
        tdengine_url="http://localhost:6041",  # 根据实际环境修改
        tdengine_user="root",
        tdengine_password="taosdata"
    )
    parser.run()