import taosrest
import psycopg2
from datetime import datetime
import binascii
import logging
import time
import uuid

# 配置日志
logging.basicConfig(
    filename='charge_order_compute_detail.log',
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    encoding='utf-8'
)


class ChargeOrderComputeDetailMigrator:
    def __init__(self):
        # TDengine 连接参数
        self.tdengine_config = {
            'host': '123.6.102.119',
            'port': 6041,
            'user': 'readonly_user',
            'password': 'Aassword123',
            'database': 'antsev'
        }

        # PostgreSQL 连接参数
        self.pg_config = {
            'host': '123.6.102.119',
            'port': 5432,
            'database': 'tms-design',
            'user': 'postgres',
            'password': '687315e66ae24eeab8bb5c0441a40d79'
        }

        self.td_conn = None
        self.td_cursor = None
        self.pg_conn = None
        self.pg_cursor = None
        self.last_processed_ts = None
        self.processed_orders = set()

    def connect(self):
        """建立与两个数据库的连接"""
        max_retries = 3
        retry_delay = 10  # 秒
        for attempt in range(max_retries):
            try:
                # 连接到 TDengine
                logging.info(f"尝试连接到 TDengine (第 {attempt + 1} 次): {self.tdengine_config}")
                rest_url = f"http://{self.tdengine_config['host']}:{self.tdengine_config['port']}"
                self.td_conn = taosrest.connect(
                    url=rest_url,
                    user=self.tdengine_config['user'],
                    password=self.tdengine_config['password'],
                    database=self.tdengine_config['database']
                )
                self.td_cursor = self.td_conn.cursor()
                logging.info("成功连接到 TDengine")

                # 测试查询以验证连接
                self.td_cursor.execute("SELECT SERVER_VERSION()")
                version = self.td_cursor.fetchone()
                logging.info(f"TDengine 服务器版本: {version[0]}")

                # 连接到 PostgreSQL
                logging.info(f"尝试连接到 PostgreSQL: {self.pg_config}")
                self.pg_conn = psycopg2.connect(
                    host=self.pg_config['host'],
                    port=self.pg_config['port'],
                    database=self.pg_config['database'],
                    user=self.pg_config['user'],
                    password=self.pg_config['password']
                )
                self.pg_conn.autocommit = True
                self.pg_cursor = self.pg_conn.cursor()
                logging.info("成功连接到 PostgreSQL")
                break  # 连接成功，退出重试循环

            except Exception as e:
                logging.error(f"连接错误 (第 {attempt + 1} 次): {str(e)}")
                if attempt < max_retries - 1:
                    logging.info(f"将在 {retry_delay} 秒后重试...")
                    time.sleep(retry_delay)
                else:
                    raise

    def parse_bcd_time(self, bcd_bytes):
        """解析 BCD 码时间（格式为 YYMMDDHHMMSS）"""
        try:
            if len(bcd_bytes) != 6:
                raise ValueError("BCD 时间字节长度必须为 6")
            year = 2000 + (bcd_bytes[0] >> 4) * 10 + (bcd_bytes[0] & 0x0F)
            month = (bcd_bytes[1] >> 4) * 10 + (bcd_bytes[1] & 0x0F)
            day = (bcd_bytes[2] >> 4) * 10 + (bcd_bytes[2] & 0x0F)
            hour = (bcd_bytes[3] >> 4) * 10 + (bcd_bytes[3] & 0x0F)
            minute = (bcd_bytes[4] >> 4) * 10 + (bcd_bytes[4] & 0x0F)
            second = (bcd_bytes[5] >> 4) * 10 + (bcd_bytes[5] & 0x0F)

            # 校验字段范围
            if not (1 <= month <= 12):
                raise ValueError(f"月份 {month} 超出范围")
            if not (1 <= day <= 31):
                raise ValueError(f"日期 {day} 超出范围")
            if not (0 <= hour <= 23):
                raise ValueError(f"小时 {hour} 超出范围")
            if not (0 <= minute <= 59):
                raise ValueError(f"分钟 {minute} 超出范围")
            if not (0 <= second <= 59):
                raise ValueError(f"秒 {second} 超出范围")

            return datetime(year, month, day, hour, minute, second)
        except Exception as e:
            logging.error(f"解析 BCD 时间出错: {str(e)}")
            return None

    def parse_hex_data(self, hex_data):
        """根据协议解析十六进制数据（23H - 最新充电订单）"""
        try:
            # 移除空格并将十六进制字符串转换为字节
            hex_bytes = bytes.fromhex(hex_data.replace(" ", ""))

            # 验证帧起始（应该是 "JX"）
            if hex_bytes[0:2] != b'JX':
                logging.warning("帧起始不是 JX，跳过")
                return None

            # 提取命令
            command = hex_bytes[2:3].hex().upper()

            # 只处理 23H 命令（最新充电订单）
            if command != '23':
                logging.warning(f"命令不是 23H (当前命令: {command})，跳过")
                return None

            # 确保 hex_data 长度足够
            if len(hex_bytes) < 154:
                logging.warning(f"hex_data 长度不足，当前长度: {len(hex_bytes)}，需要至少 154 字节")
                return None

            # 提取订单相关信息
            # 协议中（根据日志和表数据推测）：
            # - 订单编号（字节 31-46，16 个字符的 ASCII 码）
            # - 总电量（字节 119-122，分辨率 0.01 kWh）
            # - 总费用（字节 147-150，分辨率 0.01 元）
            # - 总电费（字节 147-150，分辨率 0.01 元，与总费用相同）
            # - 折扣电费（字节 151-154，分辨率 0.01 元，假设）
            # - 折扣服务费（字节 155-158，分辨率 0.01 元，假设）
            # - 充电开始时间（字节 127-132，BCD 格式）
            # - 充电结束时间（字节 133-138，BCD 格式）

            # 订单编号
            order_code = hex_bytes[30:46].decode('ascii', errors='ignore').strip()
            logging.info(f"解析订单编号: {order_code}")

            # 充电开始时间和结束时间
            start_time = self.parse_bcd_time(hex_bytes[127:133])
            end_time = self.parse_bcd_time(hex_bytes[133:139])
            logging.info(f"解析开始时间: {start_time}, 结束时间: {end_time}")

            # 总电量（字节 119-122）
            total_power = int.from_bytes(hex_bytes[119:123], byteorder='little') * 0.01  # kWh
            logging.info(f"解析总电量: {total_power} kWh")

            # 总费用（字节 147-150）
            total_money = int.from_bytes(hex_bytes[147:151], byteorder='little') * 0.01  # 元
            logging.info(f"解析总费用: {total_money} 元")

            # 总电费（假设与总费用相同，字节 147-150）
            total_elec_money = total_money  # 元
            logging.info(f"解析总电费: {total_elec_money} 元")

            # 折扣电费（字节 151-154，假设）
            discount_elec_money = int.from_bytes(hex_bytes[151:155], byteorder='little') * 0.01  # 元
            logging.info(f"解析折扣电费: {discount_elec_money} 元")

            # 折扣服务费（字节 155-158，假设）
            discount_service_money = int.from_bytes(hex_bytes[155:159], byteorder='little') * 0.01  # 元
            logging.info(f"解析折扣服务费: {discount_service_money} 元")

            # 数据合理性校验
            if total_power > 1000:  # 假设单次充电量不会超过 1000 kWh
                logging.warning(f"总电量 {total_power} 超出合理范围，跳过")
                return None
            if total_money > 1000:  # 假设单次费用不会超过 1000 元
                logging.warning(f"总费用 {total_money} 超出合理范围，跳过")
                return None

            # 初始化数据字典
            data = {
                'order_code': order_code,
                'phase_type': 4,  # 直流充电
                'rate_id': 'default',  # 假设费率 ID
                'total_money': total_money,
                'total_power': total_power,
                'total_elec_money': total_elec_money,
                'discount_elec_money': discount_elec_money,
                'discount_service_money': discount_service_money,
                'detail_start_time': start_time,
                'detail_end_time': end_time
            }

            return data

        except Exception as e:
            logging.error(f"解析十六进制数据时出错: {str(e)}")
            return None

    def migrate_data(self):
        """将新数据从 TDengine 迁移到 PostgreSQL 的 charge_order_compute_detail 表"""
        while True:
            try:
                # 如果 last_processed_ts 为空，初始化为当前时间
                if self.last_processed_ts is None:
                    try:
                        self.td_cursor.execute("SELECT ts FROM antsev.charge_jiuxing ORDER BY ts DESC LIMIT 1")
                        result = self.td_cursor.fetchone()
                        self.last_processed_ts = result[0] if result and result[0] else datetime.now()
                    except Exception as e:
                        logging.error(f"获取最新时间戳失败: {str(e)}，使用当前时间作为默认值")
                        self.last_processed_ts = datetime.now()
                    logging.info(f"初始化 last_processed_ts: {self.last_processed_ts}")

                # 查询新数据
                query = f"SELECT * FROM antsev.charge_jiuxing WHERE ts > '{self.last_processed_ts}' AND cmd = '23' ORDER BY ts"
                self.td_cursor.execute(query)
                rows = self.td_cursor.fetchall()

                if not rows:
                    logging.info("没有新订单数据，休眠 10 秒")
                    time.sleep(10)
                    continue

                for row in rows:
                    try:
                        # 从 TDengine 行中提取数据
                        timestamp = row[0]  # 时间戳
                        pile_id = row[3]    # 充电桩 ID (pile_id)
                        hex_data = row[12]  # 十六进制数据 (hex_data)

                        # 记录原始数据
                        logging.info(f"处理记录: ts={timestamp}, pile_id={pile_id}, hex_data={hex_data}")

                        # 解析十六进制数据
                        parsed_data = self.parse_hex_data(hex_data)
                        if not parsed_data:
                            logging.warning(f"无法解析 hex_data: {hex_data}，跳过此记录")
                            continue

                        # 构造唯一标识（order_code）
                        order_key = parsed_data['order_code']

                        # 检查记录是否已存在
                        check_query = """
                        SELECT 1 FROM charge_order_compute_detail WHERE order_code = %s
                        """
                        self.pg_cursor.execute(check_query, (order_key,))
                        exists = self.pg_cursor.fetchone() is not None

                        if exists or order_key in self.processed_orders:
                            logging.info(f"订单 {order_key} 已存在或已处理，跳过")
                            continue

                        # 插入 PostgreSQL 数据
                        insert_query = """
                        INSERT INTO charge_order_compute_detail (
                            uuid, connection_id, order_code, phase_type, rate_id,
                            total_money, total_power, total_elec_money,
                            discount_elec_money, discount_service_money, detail_start_time,
                            detail_end_time, created_at
                        ) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
                        """
                        # 使用 timestamp 作为 detail_start_time 和 detail_end_time 的占位符（如果解析失败）
                        start_time = parsed_data['detail_start_time'] if parsed_data['detail_start_time'] else timestamp
                        end_time = parsed_data['detail_end_time'] if parsed_data['detail_end_time'] else timestamp
                        values = (
                            str(uuid.uuid4()),  # uuid
                            pile_id,  # connection_id (假设使用 pile_id)
                            parsed_data['order_code'],
                            parsed_data['phase_type'],
                            parsed_data['rate_id'],
                            parsed_data['total_money'],
                            parsed_data['total_power'],
                            parsed_data['total_elec_money'],
                            parsed_data['discount_elec_money'],
                            parsed_data['discount_service_money'],
                            start_time,
                            end_time,
                            timestamp
                        )

                        self.pg_cursor.execute(insert_query, values)
                        self.processed_orders.add(order_key)
                        logging.info(f"插入订单 {order_key} 的记录")

                        # 记录插入的完整数据
                        log_values = {
                            'uuid': values[0],
                            'connection_id': values[1],
                            'order_code': parsed_data['order_code'],
                            'phase_type': parsed_data['phase_type'],
                            'rate_id': parsed_data['rate_id'],
                            'total_money': parsed_data['total_money'],
                            'total_power': parsed_data['total_power'],
                            'total_elec_money': parsed_data['total_elec_money'],
                            'discount_elec_money': parsed_data['discount_elec_money'],
                            'discount_service_money': parsed_data['discount_service_money'],
                            'detail_start_time': start_time,
                            'detail_end_time': end_time,
                            'created_at': timestamp
                        }
                        logging.info(f"插入到 charge_order_compute_detail 表的数据: {log_values}")

                        # 更新 last_processed_ts
                        self.last_processed_ts = max(self.last_processed_ts, timestamp)

                    except Exception as e:
                        logging.error(f"处理时间戳为 {timestamp} 的记录时出错: {str(e)}")
                        continue

            except Exception as e:
                logging.error(f"迁移过程中出错: {str(e)}")
                time.sleep(10)  # 出错后休眠 10 秒后重试

    def close(self):
        """关闭数据库连接"""
        try:
            if self.td_cursor:
                self.td_cursor.close()
            if self.td_conn:
                self.td_conn.close()
            if self.pg_cursor:
                self.pg_cursor.close()
            if self.pg_conn:
                self.pg_conn.close()
            logging.info("数据库连接已关闭")
        except Exception as e:
            logging.error(f"关闭连接时出错: {str(e)}")
            raise

    def run(self):
        """运行迁移的主方法"""
        try:
            self.connect()
            self.migrate_data()
        except Exception as e:
            logging.error(f"迁移失败: {str(e)}")
            raise
        finally:
            self.close()

if __name__ == "__main__":
    migrator = ChargeOrderComputeDetailMigrator()
    migrator.run()