import paho.mqtt.client as mqtt
import psycopg2
import json
import logging
import sys
import time
from datetime import datetime


logging.basicConfig(
    filename='mqtt_to_postgres.log',
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    encoding='utf-8'
)


MQTT_BROKER = "123.6.102.119"
MQTT_PORT = 1883
MQTT_KEEPALIVE = 60
MQTT_USERNAME = "emqx_test"
MQTT_PASSWORD = "emqx_test"
MQTT_TOPIC = "hejin/order_processed"


PG_CONFIG = {
    'host': '123.6.102.119',
    'port': 5432,
    'database': 'tms-design',
    'user': 'postgres',
    'password': '687315e66ae24eeab8bb5c0441a40d79'
}

class MqttToPostgres:
    def __init__(self):
        self.mqtt_client = None
        self.pg_conn = None
        self.pg_cursor = None
        self.setup_logging()
        self.connect_mqtt()
        self.connect_postgres()

    def setup_logging(self):
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setLevel(logging.INFO)
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        console_handler.setFormatter(formatter)
        logging.getLogger().addHandler(console_handler)

    def connect_mqtt(self):
        try:
            self.mqtt_client = mqtt.Client(client_id="MqttToPostgres", protocol=mqtt.MQTTv5)
            self.mqtt_client.username_pw_set(MQTT_USERNAME, MQTT_PASSWORD)
            self.mqtt_client.on_connect = self.on_mqtt_connect
            self.mqtt_client.on_disconnect = self.on_mqtt_disconnect
            self.mqtt_client.on_message = self.on_mqtt_message
            self.mqtt_client.connect(MQTT_BROKER, MQTT_PORT, MQTT_KEEPALIVE)
            self.mqtt_client.loop_start()
            logging.info("MQTT 客户端已初始化。")
        except Exception as e:
            logging.error(f"无法连接到 MQTT 服务器：{str(e)}")
            raise

    def on_mqtt_connect(self, client, userdata, flags, rc, properties=None):
        if rc == 0:
            logging.info("成功连接到 MQTT 服务器。")
            # 订阅主题
            self.mqtt_client.subscribe(MQTT_TOPIC, qos=1)
            logging.info(f"已订阅主题：{MQTT_TOPIC}")
        else:
            logging.error(f"连接 MQTT 服务器失败，错误码：{rc}")

    def on_mqtt_disconnect(self, client, userdata, rc, properties=None):
        logging.warning("与 MQTT 服务器断开连接。")

    def on_mqtt_message(self, client, userdata, msg):
        try:
            payload = msg.payload.decode('utf-8')
            message = json.loads(payload)
            logging.info(f"收到 MQTT 消息：{message}")

            parsed_data = message.get("order_details")
            if not parsed_data:
                logging.warning("消息中未找到 order_details，跳过。")
                return

            self.write_to_postgres(parsed_data)

        except Exception as e:
            logging.error(f"处理 MQTT 消息时出错：{str(e)}")

    def connect_postgres(self):
        max_retries = 3
        retry_delay = 10
        for attempt in range(max_retries):
            try:
                logging.info(f"尝试连接到 PostgreSQL (第 {attempt + 1} 次): {PG_CONFIG}")
                self.pg_conn = psycopg2.connect(
                    host=PG_CONFIG['host'],
                    port=PG_CONFIG['port'],
                    database=PG_CONFIG['database'],
                    user=PG_CONFIG['user'],
                    password=PG_CONFIG['password']
                )
                self.pg_conn.autocommit = True
                self.pg_cursor = self.pg_conn.cursor()
                logging.info("成功连接到 PostgreSQL")
                break

            except Exception as e:
                logging.error(f"连接 PostgreSQL 错误 (第 {attempt + 1} 次): {str(e)}")
                if attempt < max_retries - 1:
                    logging.info(f"将在 {retry_delay} 秒后重试...")
                    time.sleep(retry_delay)
                else:
                    raise

    def write_to_postgres(self, parsed_data):
        """将 parsed_data 写入 PostgreSQL 数据库"""
        try:
            # 检查记录是否已存在（基于 order_number 唯一性）
            check_query = """
            SELECT 1 FROM charge_orders WHERE order_number = %s
            """
            self.pg_cursor.execute(check_query, (parsed_data["order_number"],))
            exists = self.pg_cursor.fetchone() is not None

            if exists:
                logging.info(f"订单 {parsed_data['order_number']} 已存在，跳过插入。")
                return

            insert_query = """
            INSERT INTO charge_orders (
                order_time, gun_number, record_index, order_number, user_id, user_type, org_code,
                card_balance, vin, start_time, end_time, start_energy, end_energy, electricity_fee,
                service_fee, parking_fee, time_segments, battery_serial, created_at, updated_at
            ) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
            """
            time_segments_json = json.dumps(parsed_data["time_segments"])
            # 当前时间作为 created_at 和 updated_at
            current_time = datetime.now()

            values = (
                parsed_data["order_time"],
                parsed_data["gun_number"],
                parsed_data["record_index"],
                parsed_data["order_number"],
                parsed_data["user_id"],
                parsed_data["user_type"],
                parsed_data["org_code"],
                parsed_data["card_balance"],
                parsed_data["vin"],
                parsed_data["start_time"],
                parsed_data["end_time"],
                parsed_data["start_energy"],
                parsed_data["end_energy"],
                parsed_data["electricity_fee"],
                parsed_data["service_fee"],
                parsed_data["parking_fee"],
                time_segments_json,
                parsed_data["battery_serial"],
                current_time,
                current_time
            )

            self.pg_cursor.execute(insert_query, values)
            logging.info(f"成功插入订单 {parsed_data['order_number']} 到 PostgreSQL 数据库。")

        except Exception as e:
            logging.error(f"写入 PostgreSQL 时出错：{str(e)}")

    def run(self):
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            logging.info("程序被用户中断。")
        except Exception as e:
            logging.error(f"主循环中发生意外错误：{str(e)}")
        finally:
            self.cleanup()

    def cleanup(self):
        try:
            if self.pg_cursor:
                self.pg_cursor.close()
            if self.pg_conn:
                self.pg_conn.close()
            if self.mqtt_client:
                self.mqtt_client.loop_stop()
                self.mqtt_client.disconnect()
            logging.info("资源已清理：PostgreSQL 和 MQTT 连接已关闭。")
        except Exception as e:
            logging.error(f"关闭连接时出错：{str(e)}")

def main():
    processor = MqttToPostgres()
    processor.run()

if __name__ == "__main__":
    main()